#ifndef __MODFILE_H__
#define __MODFILE_H__

#include <string>
#include <vector>

/**
 * This parses a mod file from memory
 */
class ModFile
{
public:
	struct Instrument
	{
	public:
		std::string mName;
		int mLength;
		int mFineTune;
		int mLinearVolume;
		int mLoopStart;
		int mLoopSize;
		char *mSampleData;

		// After this point the XM Data is held but this is put in to the extended sample data table to backwards compatibility
		int mRelativeNote;
	};

	ModFile();
	virtual ~ModFile();

	void SetAddress(char *modaddr)
	{
		mAddress = modaddr;
	}

	bool ParseMod(void);
	bool XMParseMod(void);

	// The values we are most interested in
	std::string mTitle;
	int mChannels;
	std::vector<Instrument *> mInstruments;
	std::vector<int> mPatternsInSeq;
	int mEndSongJumpPosition;
	std::vector<int *> mPatternData;
	int mPatternLength;
	std::vector<int> mPatternLengths;

	bool mIsExtendedMod;

	std::vector<int> mExtendedData;

private:

	bool ParseTitle(void);
	bool ParseInstruments(void);
	bool ParsePatternTable(void);
	bool ParsePatterns(void);

	bool XMParseTitle(void);
	bool XMParseInstruments(void);
	bool XMParsePatternTable(void);
	bool XMParsePatterns(void);

	// Some utility functions
	int GetIntNS(int offset);

	unsigned short GetShort(int offset);

	unsigned short GetShortNS(int offset);

	std::string GetString(int offset,int maxlen);

	// Variables
	char *mAddress;
	int mGuessedInstruments;
	int mGuessedChannels;
	int mGuessedPatternOffset;
	int mSampleOffset;
	int mNumberPatterns;

	bool mXMUseAmigaFreqTable;
	int mXMDefaultTempo;
	int mXMDefaultBPM;
	int mXMHeaderSize;
};

#endif
