#ifndef _INC_DYNAMICARRAY
#define _INC_DYNAMICARRAY

#pragma once

template <class T, int stepsize=8> class DynamicArray
{
private:
	T *array;
	int elementsInArray;
	int usedElementsInArray;
	int stepSize;

	void SetNewSize( int newSize )
	{
		if( newSize==0 )
		{
			delete[] array;
			array = NULL;
			elementsInArray = 0;
			usedElementsInArray = 0;
			return;
		}
		if( array==NULL )
			array = new T[newSize];
		else
		{
			T* newarray = new T[newSize];
			memmove(newarray,array,sizeof(T)*elementsInArray);
			delete[] array;
			array = newarray;
		}
		elementsInArray = newSize;
	}
public:
	DynamicArray(int step = stepsize)
	{
		stepSize = step;
		array = NULL;
		elementsInArray = 0;
		usedElementsInArray = 0;
	}
	~DynamicArray()
	{
		if( array )
			delete[] array;
	}

	inline int Size(void)
	{
		return usedElementsInArray;
	}
	void Insert(T &newElement, int index )
	{
		_ASSERTE(index>=0 && index<elementsInArray);
		_ASSERTE(usedElementsInArray-index-1>=0);
		if( usedElementsInArray==elementsInArray )
			SetNewSize(elementsInArray+stepSize);
		if(usedElementsInArray-index)
			memmove(&array[index+1],&array[index],(usedElementsInArray-index)*sizeof(T));
		array[index] = newElement;
		usedElementsInArray++;
	}
	int Add(T &newElement )
	{
		if( usedElementsInArray==elementsInArray )
			SetNewSize(elementsInArray+stepSize);
		array[usedElementsInArray] = newElement;
		usedElementsInArray++;
		return usedElementsInArray-1;
	}
	T& Add(void)
	{
		if( usedElementsInArray==elementsInArray )
			SetNewSize(elementsInArray+stepSize);
		usedElementsInArray++;
		return array[usedElementsInArray-1];
	}
	void Remove( int index )
	{
		_ASSERTE(index>=0 && index<elementsInArray);
		_ASSERTE(usedElementsInArray-index-1>=0);
		if( usedElementsInArray )
		{
			if( (usedElementsInArray-index-1)>0 )
				memmove(&array[index], &array[index+1], sizeof(T)*(usedElementsInArray-index-1));
			usedElementsInArray--;
		}
	}
	void RemoveAll( void )
	{
		usedElementsInArray = 0;
	}
	inline T& operator[]( int index )
	{
		_ASSERTE( (index>=0) && (index<usedElementsInArray) );
		return array[index];
	}
	inline T& GetTop( void )
	{
		_ASSERTE(usedElementsInArray);
		return array[usedElementsInArray-1];
	}
};

#endif // _INC_DYNAMICARRAY
