/**************************************************/
/*  switch.c  switches, effects, ladders etc.     */
/**************************************************/



#include <AGB.h>
#include "asldefines.h"
#include "player.h"
#include "switch.h"
#include "switchsprite.h"
#include "rocksandstuff.h"
#include "main.h"
#include "SampleManager.h"
#include "screen.h"
#include "firejet.h"

switchparms tswitch[MAX_SWITCHES];
areaswitchparms aswitch[MAX_SWITCHES];
smashparms smash[MAX_SWITCHES];
switchparms restart[MAX_SWITCHES];
flamepitparms flamepit[MAX_SWITCHES];
ladderparms ladder[MAX_LADDERS];
tileswapparms TileSwapper[MAX_SWITCHES];
waterareaparms water[8];
modeswitchparms modeswitch[16];
l3removerparms l3remover[8];
u8 lasttileswapper;
s8 InWaterArea;
s16 Force_Scroll_xpos;
s8 Force_Scroll_dx;
u8 lastcrumble;

extern s16 game_x;
extern s16 game_y;
extern u16 BohrokEyePalette[5];
extern u8 CameraPushOveride; // set when player is being chased etc.

void HandleSwitch(void);
void HandleSmash(void);
void HandleFlamePit(void);
void HandleAirVent(void);
void HandleSlide(void);
void HandleTileSwapper(void);
void HandleTileSwapSource(void);
void HandleWaterArea(void);
void HandleAreaSwitch(void);
void HandlePickupSwitch(void);
void Handlel3Remover(void);

void HandleCrumbler(void);

void removeswitch(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	tswitch[me].state = 0;	
	tswitch[me].xpos = 0;
}

void initswitch(void)
{
	u8 i;
	u8 me;
	
	Object[num_strats].handle = HandleSwitch;
	Object[num_strats].remover = removeswitch;
	me = Object[num_strats].parameter % 100;
	tswitch[me].hide = (Object[num_strats].parameter) / 100;
	tswitch[me].frame = GrabASprite();
	i = tswitch[me].frame;
      	Sprite[i].CharNo = SWITCH_SPRITE;
    //   	Sprite[i].Shape = OAM_SQUARE;
   		Sprite[i].Size = 2; //OAM_SIZE_16x16;
   		Sprite[i].Pltt = 9;
         Sprite[i].Priority = 1;
        tswitch[me].xpos = parm_xpos;
        tswitch[me].ypos = parm_ypos;
        tswitch[me].state = 0;
        Object[num_strats].parameter = me;
}

void HandleSwitch(void)
{
	u8 me;
	s16 sprx,spry;
	me = Object[current_strat].parameter;
	mysprite = tswitch[me].frame;
	
	if(tswitch[me].hide > 0)
		if(tswitch[tswitch[me].hide].state == 0)
		{	
			Sprite[mysprite].VPos = 160;
			return;
		}
	sprx = tswitch[me].xpos - game_x;
	spry = tswitch[me].ypos - game_y;
	if(sprx >= -32 && sprx < 240 && spry >= -32 && spry <= 160)
	{
		Sprite[mysprite].CharNo = SWITCH_SPRITE; // + tswitch[me].state * 4;
		Sprite[mysprite].HPos = sprx & 511;
		Sprite[mysprite].VPos = (spry + 256) & 255;
		LoadSprite((u32)SwitchSprite + tswitch[me].state * SWITCH_SPRITE_SIZE,SWITCH_SPRITE_OFFSET,SWITCH_SPRITE_SIZE);
		
		
				// let's make it a platform :)
		if(tswitch[me].state == 0)
		{
		
			if(collision_underme > tswitch[me].ypos - 16 &&  player.ypos <= (tswitch[me].ypos + 16) && player.ypos >= tswitch[me].ypos - 18)
			{

				if((player.xpos > tswitch[me].xpos - 2 ) && (player.xpos < tswitch[me].xpos + 22))
				{
					collision_underme = tswitch[me].ypos - 13;
					platform_underme = collision_underme;
					
					if(player.ypos >= tswitch[me].ypos - 18)
					{
					
						if(player.jump >= 0)
						{
							if(player.jump > 0)
							{
								if(WhatPlayer == 1) // chief
								{
									tswitch[me].state = 1; // switch
									Sample_Play(SMPL_SWITCH);
								}

							}
							player.jump = 0;
							player.ypos = tswitch[me].ypos - 13 ;
							player.falltime = 0;
						
							
						//	Sprite[mysprite].VPos+=4;
						}
					}
				}
			}
		}
		
	}
	else
	{
		Sprite[mysprite].VPos = 160;
	}
}


void initareaswitch(void)
{
	u32 i;
	u8 me;
	
	me = Object[num_strats].parameter;
	Object[num_strats].handle = HandleAreaSwitch;
	Object[num_strats].remover = removeswitch;
    
    if(wp_xpos == -1)
    {
    	aswitch[me].xpos = parm_xpos - 64;
    	aswitch[me].ypos = parm_ypos - 64;
    	aswitch[me].w = (128 >> 3);
    	aswitch[me].h = (128 >> 3);
    }
    else
    {  
	    aswitch[me].xpos = parm_xpos;
	    aswitch[me].ypos = wp_ypos;
	    aswitch[me].w = (wp_xpos - parm_xpos) >> 3;
	    aswitch[me].h = (parm_ypos - wp_ypos) >> 3;
    }
    aswitch[me].state = 0;
}

void HandleAreaSwitch(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	
	if(player.xpos > aswitch[me].xpos && player.xpos < (aswitch[me].xpos + (aswitch[me].w << 3)) && player.ypos > aswitch[me].ypos && player.ypos < (aswitch[me].ypos + (aswitch[me].h << 3)))
		tswitch[me].state = 1;

}



void initpickupswitch(void)
{
	u32 i;
	u8 me;
	
	me = (Object[num_strats].parameter % 100);
	
	tswitch[me].frame = max(1,Object[num_strats].parameter / 100);
	Object[num_strats].handle = HandlePickupSwitch;
	Object[num_strats].remover = removeswitch;
    Object[num_strats].parameter = Object[num_strats].parameter % 100;
   	tswitch[me].xpos = parm_xpos;
   	tswitch[me].ypos = parm_ypos;
    tswitch[me].state = 0; 
}

void HandlePickupSwitch(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	
	if(PickupItems >= tswitch[me].frame)
		tswitch[me].state = 1;

}



u8 NextSmash(void)
{
	u8 a;
	for(a = 0; a < MAX_SWITCHES;a++)
	{
		if(smash[a].xpos == 0)
			return a;
	}
	return 0;
}

void RemoveSmash(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	smash[me].xpos = 0;
}

void initsmash(void)
{
	u32 i;
	u8 me;
	
	me = NextSmash();
	Object[num_strats].handle = HandleSmash;
	Object[num_strats].remover = RemoveSmash;
        smash[me].xpos = parm_xpos;
        smash[me].ypos = wp_ypos;
        smash[me].xpos2 = wp_xpos;
        smash[me].ypos2 = parm_ypos;
        smash[me].state = Object[num_strats].parameter % 100; // switch number
        smash[me].hits = 3;
        smash[me].type = Object[num_strats].parameter / 100; // 0 = wall, 1 = floor, 2 = wall blocker, 3 = floor blocker
       	smash[me].lastaction = 0;
       Object[num_strats].parameter = me;
}

void HandleSmash(void)
{
	u8 me,a;
	s16 sprx,spry;
	s16 px;
	
	me = Object[current_strat].parameter;
		sprx = smash[me].xpos - game_x;
		spry = smash[me].ypos - game_y;
		
		
		if(smash[me].type & 2)
			if(tswitch[smash[me].state].state)
			{
				FlashMapColour(236,31,31,31);
				Sample_Play(SMPL_WALL_SMASH01);
				for(a = 0; a < 10 ;a ++)
				{
					spawn(initrubble,smash[me].xpos + rnd(smash[me].xpos2 - smash[me].xpos),smash[me].ypos + rnd(smash[me].ypos2 - smash[me].ypos),0);
					spawnparticle(smash[me].xpos + rnd(smash[me].xpos2 - smash[me].xpos),smash[me].ypos + rnd(smash[me].ypos2 - smash[me].ypos),srnd(16),-rnd(64),0,1,64,50,0);


				}
				remove();	
			}
		if(sprx >= -160 && sprx < 320 && spry >= -32 && spry <= 160)
		{
			// let's make it a platform :)
			if(smash[me].type & 1)
			{
					if(collision_underme > smash[me].ypos && player.ypos >= smash[me].ypos && player.ypos <= (smash[me].ypos + 32))
				{

					if((player.xpos > smash[me].xpos ) && (player.xpos < smash[me].xpos2))
					{
						collision_underme = smash[me].ypos + 4;
						platform_underme = collision_underme;
					}
				}
			}
			else
			{
				if(player.ypos > (smash[me].ypos) && player.ypos < (smash[me].ypos2 + 16))  // check for left/right blocking
				{
					if(player.xpos < smash[me].xpos + 16 && player.xpos > smash[me].xpos - 16 )
						if(player.dx > 0)
						{
							player.dx = 0;
							player.xpos = smash[me].xpos - 16;
						}
					if(player.xpos > smash[me].xpos2 - 16 && player.xpos < smash[me].xpos2 + 16)
						if(player.dx < 0)
						{
							player.dx = 0;
							player.xpos = smash[me].xpos2 + 16;
						}
				}
			}

			if(smash[me].type < 2)
			{
				px = player.xpos;
				if(Sprite[player.frame].HFlip == 0)
					px += 64;
				else
					px -=64;
				if(max(player.xpos,px) >= smash[me].xpos && min(player.xpos,px) < smash[me].xpos2 && player.ypos >= smash[me].ypos && player.ypos < (smash[me].ypos2))
				{

					
					if(player.ActionFlags & ACTION_POWER)
					{
						if(smash[me].lastaction == 0) // player special powered on us
						{
							smash[me].hits--;
							FlashMapColour(236,31,31,31);
							smash[me].lastaction = 1;
							if(smash[me].hits == 0 || WhatPlayer == 1)
							{
							
								Sample_Play(SMPL_WALL_SMASH01);
								for(a = 0; a < 10 ;a ++)
								{
									spawn(initrubble,smash[me].xpos + rnd(smash[me].xpos2 - smash[me].xpos),smash[me].ypos + rnd(smash[me].ypos2 - smash[me].ypos),0);
								}
							
								tswitch[smash[me].state].state = 1; // throw linked switch...
								remove();
							}
						}
					}
					else
					{
						smash[me].lastaction = 0;
					}
				}
			}
	
			

		}
		
}

u8 GetNextFlame(void)
{
	u8 a;
	for(a = 0; a < MAX_SWITCHES;a++)
	{
		if(flamepit[a].xpos == 0)
			return a;
	}
	return 0;

}


void RemoveFlamePit(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	flamepit[me].xpos = 0;
}

void initflamepit(void)
{
	u8 me;
	
	me = GetNextFlame();
	flamepit[me].xpos = parm_xpos;
	flamepit[me].ypos = parm_ypos;
	flamepit[me].time = (Object[num_strats].parameter >> 4) * 15;
	flamepit[me].time2 = (Object[num_strats].parameter & 15) * 15 + flamepit[me].time;
	flamepit[me].clock = 0;	
	Object[num_strats].handle = HandleFlamePit;
	Object[num_strats].remover = RemoveFlamePit;
	Object[num_strats].parameter = me;
}

void HandleFlamePit(void)
{
	u8 me;
	s16 sprx,spry;
	u8 flameon;
	u32 sprite_ptr;
	
	me = Object[current_strat].parameter;

	if(abs(player.xpos - flamepit[me].xpos) > 400)
		return;

	flamepit[me].clock++;
	flameon = 0;
	if(flamepit[me].clock >= flamepit[me].time) // flame on
	{	
		flameon = 1;
		if(flamepit[me].clock > flamepit[me].time2) // flame off
			flamepit[me].clock = 0;
	}
	if(abs(player.xpos - flamepit[me].xpos) < 16 && player.ypos > flamepit[me].ypos - 64 && player.ypos < flamepit[me].ypos + 56 && flameon == 1)
	{
		HitPlayer(1);
	}

	sprx = flamepit[me].xpos - game_x - 8;
	spry = flamepit[me].ypos - game_y - 16 - flameon * 48;
	
	if(sprx > -64 && sprx < 240 && spry > -64 && spry < 160 && flameon == 1  && (flamepit[me].clock & 1) == (me & 1))
	{
	//	if((flamepit[me].clock + me) & 1)
		{
			mysprite = NextSprite; 
	   		
	   		NextSprite = NextSprite + 1;
	   		
	   		if(NextSprite == mysprite)
	   		{
	   			// big up maths bug
	   			
		   		Sprite[mysprite] = *(OamData *)OamData_Sample;
	   		Sprite[mysprite].AffineMode = 0;
   		}
	   		
	   		Sprite[mysprite] = *(OamData *)OamData_Sample;
	   		Sprite[mysprite].AffineMode = 0;
	   		if(flameon)
	   		{
		   		Sprite[mysprite].Size = FireJetSizeShape[0];
		   		Sprite[mysprite].Shape = FireJetSizeShape[1];
				sprite_ptr = (u32)FireJetSprite;
				DmaCopy(3,sprite_ptr,(u32)(OBJ_MODE0_VRAM + EnemySpriteOffset[5]),1024,16);  
				DmaWait(3)	
//				LoadEnemySprite(sprite_ptr, 1024);
	    	}
	    	else
	    	{
	    		Sprite[mysprite].Size = FireJet2SizeShape[0];
	   			Sprite[mysprite].Shape = FireJet2SizeShape[1];
	 			sprite_ptr = (u32)FireJet2Sprite;
				DmaCopy(3,sprite_ptr,(u32)(OBJ_MODE0_VRAM + EnemySpriteOffset[5]),128,16);  
				DmaWait(3)	
// 			LoadEnemySprite(sprite_ptr,128);
	  		}
	    	Sprite[mysprite].Priority = 2;
			
DmaCopy(3,(u32)FireJetPalette,(u32)(OBJ_PLTT + EnemyPaletteOffset[4]),32,16); 
DmaWait(3); 
//			LoadEnemyPalette(FireJetPalette,nxtenemypal);
	   		Sprite[mysprite].Pltt = 6;
			Sprite[mysprite].HFlip = 0; 
			
			Sprite[mysprite].CharNo = EnemySprite[5];
			Sprite[mysprite].HPos = sprx & 511;
			Sprite[mysprite].VPos = (spry + 256) & 255;
		//	if((flamepit[me].clock + me) & 1)
		//		Sprite[mysprite].VPos = 160;
		}
	}
	
}

void initairvent(void)
{
	u8 me;
	
	me = GetNextFlame();  // air vents use flame pit array
	flamepit[me].xpos = parm_xpos;
	flamepit[me].ypos = parm_ypos;
	flamepit[me].time = (Object[num_strats].parameter / 100) * 6;
	flamepit[me].time2 = (Object[num_strats].parameter % 100) * 6;
	if(flamepit[me].time == 0 && flamepit[me].time2 == 0)
	{
		flamepit[me].time = 0;
		flamepit[me].time2 = 64;
	}
	flamepit[me].clock = 0;	
	flamepit[me].height = ( parm_ypos - wp_ypos) >> 3;
	Object[num_strats].handle = HandleAirVent;
	Object[num_strats].remover = RemoveFlamePit;
	Object[num_strats].parameter = me;
	
}

void HandleAirVent(void)
{
	u8 me;
	s16 sprx;
	
	me = Object[current_strat].parameter;
	
	if(abs(player.xpos - flamepit[me].xpos) > 400)
		return;

	flamepit[me].clock++;
	if(flamepit[me].clock >= flamepit[me].time) // flame on
	{
		if(flamepit[me].clock > flamepit[me].time2) // flame off
			flamepit[me].clock = 0;
	
		sprx = flamepit[me].xpos - game_x;
		
		if(sprx > -64 && sprx < 304)
		{
			if((flamepit[me].clock & 3) == 0)
				spawnparticle(flamepit[me].xpos + srnd(32),max(flamepit[me].ypos - (flamepit[me].height << 3),min(flamepit[me].ypos,game_y + 160)),0,-8,-99,-1,64,6,1);	
		
		
			if(abs(player.xpos - flamepit[me].xpos) < 64 && player.ypos > flamepit[me].ypos - (flamepit[me].height << 3) && player.ypos < flamepit[me].ypos + 48)
			{
				player.falltime = 0;
				if(GameClock & 8)
					player.ypos -= 3;
				if(player.jump > 0)
					player.jump -= 3;
			}
		}
	}	
}

u8 GetNextRestart(void)
{
	u8 a;
	for(a = 0; a < MAX_SWITCHES;a++)
	{
		if(restart[a].xpos == 0)
		{
			return a;
		}
	}
	return 0;
}

void RemoveRestartPoint(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	restart[me].xpos = 0;
}

void initrestartpoint(void)
{
	u8 me;
	
	me = GetNextRestart();  // air vents use flame pit array
	restart[me].xpos = parm_xpos;
	restart[me].ypos = ForceOtherCollision(parm_xpos,parm_ypos);

	Object[num_strats].handle = HandleRestartPoint;
	Object[num_strats].remover = RemoveRestartPoint;
	Object[num_strats].parameter = me;
}

void HandleRestartPoint(void)
{
	u8 me;
	u32 sprite_ptr;
	s16 sprx,spry;
	u8 a,b;
	
	me = Object[current_strat].parameter;

	a = 5;
	b = 0;
	if(restart_xpos == restart[me].xpos)
	{
		a = 4;
		b = 16;
	}
#if 1
	
	{
		if(restart[me].xpos - game_x > -32 && restart[me].xpos - game_x < 240 && a == 4 && (GameClock & 2))
		{
		//	if(GameClock & 2)
			spawnparticle(restart[me].xpos + (fsin[((GameClock << 2) + 64) & 255] >> a),restart[me].ypos - 32  + (fsin[(GameClock << 2) & 255] >> 5),0,0,0,0,16 + b,7,0);
			
		}
	}
#endif
	sprx = restart[me].xpos - game_x - 16;
	spry = restart[me].ypos - game_y - 32;
	if(sprx > -32 && sprx < 240 && spry > -32 && spry < 160)
	{
		mysprite = NextSprite++; // Bohrok[me].frame;
    	Sprite[mysprite] = *(OamData *)OamData_Sample;
   		
   		Sprite[mysprite].AffineMode = 0;
   		Sprite[mysprite].Size = SwitchSizeShape[0];
   		Sprite[mysprite].Shape = SwitchSizeShape[1];
    	Sprite[mysprite].Priority = 2;
		sprite_ptr = SwitchSprite + 512 * 2;
		LoadEnemySprite(sprite_ptr, 512);
		LoadEnemyPalette(SwitchPalette,nxtenemypal);
		if(restart_xpos == restart[me].xpos)
		{
			LoadEnemyCycle2Palette(&BohrokEyePalette[(GameClock >> 2) % 3],nxtenemypal);
	   		spry -= bobtable[GameClock & 31];
	   	}
	   	Sprite[mysprite].Pltt = 2 + PalCache;
		Sprite[mysprite].CharNo = EnemySprite[SpriteCache];
		Sprite[mysprite].HPos = sprx & 511;
		Sprite[mysprite].VPos = (spry + 256) & 255;
	}

	if(abs(player.xpos - restart[me].xpos) < 32 && abs(player.ypos - restart[me].ypos) < 64)
	{
		restart_xpos = restart[me].xpos;
		restart_ypos = restart[me].ypos;
	}
}

u8 GetNextLadder(void)
{
	u8 a;
	for(a = 0; a < MAX_LADDERS; a++)
		if(ladder[a].xpos == 0)
			return a;
	return 0;
}

void RemoveLadder(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	ladder[me].xpos = 0;
}


void initladder(void)
{
	u8 me;
	
	me = GetNextLadder();  // air vents use flame pit array
	ladder[me].xpos = parm_xpos;
	ladder[me].ypos = parm_ypos;
	ladder[me].width = (wp_xpos - parm_xpos) >> 3;
	ladder[me].height = (parm_ypos - wp_ypos + 16) >> 3;
	
	Object[num_strats].handle = HandleLadder;
	Object[num_strats].remover = RemoveLadder;
	Object[num_strats].parameter = me;
}

void HandleLadder(void)
{
	u8 me;
	u16 w,h;
	me = Object[current_strat].parameter;



	w = (u16)(ladder[me].width << 3);
	h = (u16)(ladder[me].height << 3);
	if(player.xpos >= ladder[me].xpos && player.xpos <= (ladder[me].xpos + w))
		if(player.ypos <= ladder[me].ypos && player.ypos >= (ladder[me].ypos - h))
		{
			if(collision_left_ladder) // ladder wall left
			{
				player.ActionFlags |= ACTION_LADDERLEFT;

			}
			if(collision_right_ladder) // ladder wall right
			{
				player.ActionFlags |= ACTION_LADDERRIGHT;
			}
		}	
		
}


void initslide(void)
{
	u8 me;
	
	me = GetNextLadder();  // slides use ladder array
	ladder[me].xpos = parm_xpos;
	ladder[me].ypos = parm_ypos;
	ladder[me].width = (wp_xpos - parm_xpos) >> 3;
	ladder[me].height = (parm_ypos - wp_ypos) >> 3;
	
	Object[num_strats].handle = HandleSlide;
	Object[num_strats].remover = RemoveLadder;
	Object[num_strats].parameter = me;
}

void HandleSlide(void)
{
	u8 me;
	u16 w,h;
	u8 a;
	
	a = 2;
	if((GameClock & 7) == 0) 
		a = 3;
	me = Object[current_strat].parameter;
	w = (u16)(ladder[me].width << 3);
	h = (u16)(ladder[me].height << 3);
	if(player.xpos >= ladder[me].xpos && player.xpos <= (ladder[me].xpos + w))
		if(player.ypos <= ladder[me].ypos && player.ypos >= (ladder[me].ypos - h))
		{
			if(player.ypos >= (collision_underme - 8)) // on floor)
			{
				player.ActionFlags |= ACTION_SLIDE;
				if(ForceOtherCollision(player.xpos - 8,player.ypos) < collision_underme) // slope up left
				{
					if(controller & pad_left && player.dx > -14)
						a >>= 1;
					player.dx+=a;
					if(player.dx > 48)
						player.dx = 48;
				}
				if(ForceOtherCollision(player.xpos + 8,player.ypos) < collision_underme) // ladder wall right
				{
					if(controller & pad_right && player.dx < 14)
						a >>= 1;
					player.dx-=a;
					if(player.dx < -48)
						player.dx = -48;
				}
			}
		}	
		
}

void initcrumbler(void)
{
	u8 me;
	
	me = GetNextLadder();  // slides use ladder array
	ladder[me].xpos = parm_xpos;
	ladder[me].ypos = wp_ypos + 8;
	ladder[me].width = (wp_xpos - parm_xpos) >> 3;
	ladder[me].height = Object[num_strats].parameter; // scroll speed
	Object[num_strats].handle = HandleCrumbler;
	Object[num_strats].remover = RemoveLadder;
	Object[num_strats].parameter = me;
	Force_Scroll_xpos = 0;
	Force_Scroll_dx = 0;
}

void HandleCrumbler(void)
{
	u8 me;
	u16 w,h;
	u8 a;
	s16 x,y;
	
	me = Object[current_strat].parameter;
	w = (u16)(ladder[me].width << 3);
	game_y = max(8,game_y);
	if(ladder[me].height)
	{
		if(player.xpos > 260)
		{
			if(Force_Scroll_xpos == 0)
			{
				Force_Scroll_xpos = game_x * 2;
				Force_Scroll_dx = ladder[me].height;
				CameraPushOveride = 1;
			}	
		}
	}
	
	if(player.xpos >= ladder[me].xpos && player.xpos <= (ladder[me].xpos + w) && player.xpos > Force_Scroll_xpos / 2 + 8)
	{
	//	if(player.ypos >= ladder[me].ypos - 16) // stood on crumbler box
		{
			x = player.xpos >> 3;
			y = (player.ypos >> 3);
			if(WorkMap[y * PlayerMapWidth + x] > 0)
			{
				y-= 2;
				collision_underme = ladder[me].ypos + 1;
				platform_underme = collision_underme;
				if(player.ypos >= ladder[me].ypos && player.ypos < ladder[me].ypos + 32)
				{

					player.ypos = collision_underme;
					if(player.jump > 0)
					{
						player.jump = 0;
						player.ypos = ladder[me].ypos + 1;
						player.falltime = 0;
					
					//	Sprite[mysprite].VPos+=4;
					}
				}
				if(WorkMap[x] == 0)
				{
					WorkMap[x] = 1; // start collapse
					spawn(initrubblefall,x * 8 + rnd(8),player.ypos + rnd(8),0);
					if(lastcrumble == 0)
					{
						Sample_Play(SMPL_WALL_SMASH01);
						lastcrumble = 15 + rnd(15);
					}
				}
			}
		}	
	}	

	if(player.xpos >= ladder[me].xpos  && player.xpos < ladder[me].xpos + w + 64)
	{
		for(x = (ladder[me].xpos >> 3); x < ((ladder[me].xpos + w) >> 3); x++)
		{
			if(WorkMap[x] > 0)
			{
				WorkMap[x]++;
				if(WorkMap[x] > 30)
				{
					WorkMap[x] = 0;
					y = (ladder[me].ypos >> 3) - 2;
					for(a = y; a < PlayerMapHeight; a++)
					{
						if(WorkMap[a * PlayerMapWidth + x] > 0)
						{
							WorkMap[a * PlayerMapWidth + x] = 0;
							spawn(initrubblefall,x * 8 + rnd(8),a * 8 + rnd(8),0);
						}
					}
				}
			}
		}
	//	if(player.xpos > ladder[me].xpos + w + 32 && ladder[me].height == 0)
	//		remove();

	}
}
u8 NextTileSwapper(void)
{
	u8 a;
	for(a = 0; a < 16; a++)
	{
		if(TileSwapper[a].xpos == 0)
			return a;
	}
	return 0;
}

void RemoveTileSwapper(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	TileSwapper[me].xpos = 0;
}

void inittileswapper(void)
{
	u8 me;
	me = NextTileSwapper();
	TileSwapper[me].switchnum = Object[num_strats].parameter;
	TileSwapper[me].xpos = (parm_xpos / 8);
	TileSwapper[me].ypos = (parm_ypos / 8 );
	TileSwapper[me].width = (wp_xpos >> 3) - (parm_xpos >> 3);
	TileSwapper[me].height = (parm_ypos >> 3) - (wp_ypos >> 3);
	TileSwapper[me].ypos -= TileSwapper[me].height;
	
	Object[num_strats].handle = HandleTileSwapper;
	Object[num_strats].remover = RemoveTileSwapper;
	Object[num_strats].parameter = me;
	lasttileswapper = me;
}

void HandleTileSwapper(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	if(tswitch[TileSwapper[me].switchnum].state) // activate
	{
		s16 x,y;
		u8 w,h;
		s16 xpos,ypos,sourcex,sourcey;
		// swap tiles from sourcex,sourcey, width,height to xpos,ypos, width, height	
		w = TileSwapper[me].width;
		h = TileSwapper[me].height;
		xpos = TileSwapper[me].xpos;
		ypos = TileSwapper[me].ypos;
		sourcex = TileSwapper[me].sourcex;
		sourcey = TileSwapper[me].sourcey;
		last_map_x = -1; // force map to refresh
		for(y = 0 ; y < h; y++)
		{
			for(x = 0; x < w; x++)
			{
				WorkMap[xpos + x + (ypos + y) * PlayerMapWidth] = WorkMap[sourcex + x + (sourcey + y) * PlayerMapWidth];
			}
		}	
		remove();
	
	}
}

void inittileswapsource(void) // 2nd pass only
{
	u8 a;
	for(a = 0; a < 16; a++)
	{
		if((TileSwapper[a].xpos > 0) && (TileSwapper[a].switchnum == Object[num_strats].parameter))
		{
			TileSwapper[a].sourcex = (parm_xpos / 8);
			TileSwapper[a].sourcey = (parm_ypos / 8);
		}
	}
	Object[num_strats].handle = HandleTileSwapSource;
}

void HandleTileSwapSource(void)
{
	
	remove();
}

u8 NextWaterArea(void)
{
	u8 a;
	for(a = 0; a < 8; a++)
		if(water[a].xpos == 0)
			return a;
	return 0;
}

void RemoveWaterArea(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	water[me].xpos = 0;
}

void initwaterarea(void)
{
	u8 me;
	me = NextWaterArea();
	
	Object[num_strats].handle = HandleWaterArea;
	Object[num_strats].remover = RemoveWaterArea;
	Object[num_strats].parameter = me;
	
	if(parm_ypos > wp_ypos)
	{
		water[me].ypos = wp_ypos;
		water[me].bry = parm_ypos;
	}
	else
	{
		water[me].ypos = parm_ypos;
		water[me].bry = wp_ypos;
	}
	
	water[me].xpos = parm_xpos;
	water[me].brx = wp_xpos;
	InWaterArea = -1;
}

void HandleWaterArea(void)
{

	u8 me;
	me = Object[current_strat].parameter;
	if(player.xpos > water[me].xpos && player.xpos < water[me].brx && player.ypos > water[me].ypos + 9 && player.ypos < water[me].bry)
	{
		if(player.jump >= 0)
		{

			if(InWaterArea == -1)
			{
				Sample_Play(SMPL_WATERSPLASH01);
				SpawnParticleSplash(player.xpos,player.ypos,5);
			}
			InWaterArea = me;
		//	WaterEffect(8,10);
			
			if(player_state != player_state_notatalldead)
			{	
				if(player_state != player_state_swimming)
				{
					Sample_Play(SMPL_WATERSPLASH01);
				}
				player_state = player_state_swimming;
			}
			WaterLevel = water[me].ypos + 9;
			TorchLit = 0; // put out torch when in water
			Screen1(1);
			WaterEffect(13,12);
		}
	}
	else
	{
		if(InWaterArea == me)
		{
			InWaterArea = -1;
			WaterLevel = 32767;
		}
		if(game_x > water[me].xpos - 240 && game_x < water[me].brx && player.ypos < water[me].ypos + 9)
			WaterLevel = water[me].ypos + 9;
	}
}

u8 Nextl3Remover(void)
{
	u8 a;
	for(a = 0; a < 8;a++)
		if(l3remover[a].xpos == 0)
			return a;
	return 0;
}

void Removel3Remover(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	l3remover[me].xpos = 0;
}

void initl3remover(void)
{
	u8 me;
	me = Nextl3Remover();
	
	Object[num_strats].handle = Handlel3Remover;
	Object[num_strats].remover = Removel3Remover;
	l3remover[me].xpos = parm_xpos;
	l3remover[me].ypos = parm_ypos;
	l3remover[me].w = (wp_xpos - parm_xpos) >> 3;
	l3remover[me].h = (parm_ypos - wp_ypos) >> 3;
	l3remover[me].tswitch = 	Object[num_strats].parameter; 
	Object[num_strats].parameter = me;
}

void Handlel3Remover(void)
{
	u8 me;
	u8 x,y;
	me = Object[current_strat].parameter;
	
	if(tswitch[l3remover[me].tswitch].state) // switch thrown
	{
	//	for(x = 0; x < l3remover[me].w;x++)
	//		for(y = 0; y < l3remover[me].h;y++)
	//			FrontMap[l3remover[me].xpos + x + (l3remover[me].ypos + y) * PlayerMapWidth] = 0;
		remove();			
	}
}


u8 NextModeSwitch(void)
{	
	u8 a;
	for(a = 0; a < 16; a++)
		if(modeswitch[a].xpos == 0)
			return a;
	return 0;
}

void RemoveModeSwitch(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	modeswitch[me].xpos = 0;
}

void initmodeswitch(void)
{
	u8 me;
	me = NextModeSwitch();
	
	Object[num_strats].handle = HandleModeSwitch;
	Object[num_strats].remover = RemoveModeSwitch;
	modeswitch[me].xpos = parm_xpos;
	modeswitch[me].ypos = parm_ypos;
	modeswitch[me].mode = Object[num_strats].parameter; // mode 0 = non transparent, 1 = shading/lighting, 2 = water, 3= layer off
	Object[num_strats].parameter = me;
}

void HandleModeSwitch(void)
{
	u8 me;
	me = Object[current_strat].parameter;
	if(player.xpos > (modeswitch[me].xpos - 240) && player.xpos < (modeswitch[me].xpos + 240))
	{
		if(player.xpos < modeswitch[me].xpos)
			ForegroundMode = modeswitch[me].mode / 10;
		else
			ForegroundMode = modeswitch[me].mode % 10;
		//ForegroundMode = 3;
		
		switch(ForegroundMode)
		{
			case 0:	
				Screen1(1);
				Screen1Base(0);
				BlendAlpha(5,8);
				break;
			case 1:
				Screen1(1);
				Screen1Base(0);
 				TextColour(31,31,31);
				WaterEffect(8,15);
				break;
			case 2:
				Screen1(1);
				Screen1Base(0);
				WaterEffect(13,12);
				break;
			case 3:
				Screen1(0); // switches screen 1 off
				Screen1Base(0);
				break;
			case 4: // use main map layer tiles
				Screen1(1);
				Screen1Base(1);
				break;
			case 5:	// heat haze
				Screen1(1);
				Screen1Base(0);
				WaterEffect(7,14);
				break;
		}
	}	
}