/******************************************/
/*										  */
/*  ASL Defines for GBA					  */
/* Aaron Fothergill, Argonaut Software    */
/* Oct 2001								  */
/******************************************/

#include <AGB.h>
#include "asldefines.h"
#include "playermap.h"
#include "player.h"
#include "main.h"
#include "menu.h"
#include "le_wahi1.h" // used for rnd function

extern u8 Cheatmode;
extern u8 Game_Level;
extern u8 Game_Character;
extern u16 SaveGame[64]; // save game buffer
extern u8 GameSlot[2];

u8 warnobject;
u16 SpriteCacheOverrun;
u8 SpriteCacheCount;

// controller globals

u16 controller,  controller_push;
u16 current_strat,num_strats;
object_type Object[MAX_STRATS];
s32 parm_xpos,parm_ypos;
s32 wp_xpos,wp_ypos;

OamData  Sprite[128];                // OAM backup
OamData	 TSprite[6]; // temp sprites

u16	collision_underme;		// nearest ground point beneath player
u8 collision_left;			// floor of left collision	
u8 collision_right;		// floor of right collision
u8 collision_left_ladder;
u8 collision_right_ladder;

u16 collision_floornum;		// floor number of last collision returned
u16 collision_aboveme;   // nearest ceiling point above player
u16 platform_underme;		// nearest platform beneath player
u16 platform_underme2;  	// resiliant version
u16 Floor_Depth;
u32 RandomClock;			// rnd function clock
u32 RandomClock2;
u16 * col_map;
u8 * col_tile_fine;
u32 CurrentPalette;

u16 * flash_ptr;
u8 flash_time;
u16 flash_old_bgr;
u8 IsThePlayer;

const s16 fsin[256] = {0,6,12,18,25,31,37,43,50,56,62,68,74,80,86,92,98,104,109,115,121,126,132,137,142,147,153,
				158,162,167,172,177,181,185,190,194,198,202,206,209,213,216,220,223,226,229,231,234,237,
				239,241,243,245,247,248,250,251,252,253,254,254,255,255,256,256,255,255,255,254,253,253,
				252,250,249,248,246,244,242,240,238,235,233,230,227,224,221,218,215,211,208,204,200,196,
				192,188,184,179,175,170,165,160,155,150,145,140,135,129,124,118,113,107,101,95,89,83,77,
				71,65,59,53,47,41,34,28,22,16,9,3,-3,-10,-16,-22,-28,-35,-41,-47,-53,-60,-66,-72,-78,-84,
				-90,-96,-101,-107,-113,-118,-124,-130,-135,-140,-146,-151,-156,-161,-166,-170,-175,-180,
				-184,-188,-193,-197,-201,-205,-208,-212,-215,-219,-222,-225,-228,-231,-234,-236,-239,-241,
				-243,-245,-247,-248,-250,-251,-252,-253,-254,-255,-256,-256,-256,-257,-257,-256,-256,-256,
				-255,-254,-253,-252,-251,-249,-248,-246,-244,-242,-240,-238,-235,-233,-230,-227,-224,-221,
				-218,-214,-211,-207,-203,-199,-195,-191,-187,-183,-178,-174,-169,-164,-159,-154,-149,-144,
				-139,-133,-128,-122,-117,-111,-105,-100,-94,-88,-82,-76,-70,-64,-58,-51,-45,-39,-33,-26,
				-20,-14,-8,-1};  

// Sprite List stuff
u8 NextSprite;		// next sprite to be used
u8 LastSprite;      // Last sprite used in previous frame
u8 mysprite;

u16 pad_A;
u16 pad_B;
u16 pad_L;
u16 pad_R;

u8 cpushedtime,cpushedflag;

u32 SpriteCacheSource[6];
u32 SpriteCacheSize[6];
u32 PalCacheSource[6];
u32 PalCacheFlash[6];
u8 PalCacheFlashOff[6];


/*------------------------------------------------------------------*/
/*                      Key read in                                 */
/*------------------------------------------------------------------*/

void KeyRead(void)
{
    u16 ReadData = (*(vu16 *)REG_KEYINPUT ^ 0x03ff);
    controller_push = ReadData & (ReadData ^ controller); // trigger input
	// if in menus allow controller_push to refresh after delay
	if(GameStateLoop == MenuLoop)
	{
		if(ReadData & pad_D) // if pushed
		{
			cpushedtime++;
			if(cpushedtime > (30 - cpushedflag * 20))
			{
				cpushedtime = 0;
				cpushedflag = 1;
				controller_push = ReadData;
			}
		}
		else
		{
			cpushedtime = 0;
			cpushedflag = 0;
		}
	}
   
    controller = ReadData;                                // current input
}

void NoCollision(void)
{
}

void spawn(Funcp stratinit,s16 xpos,s16 ypos,s32 parameter)
{
	u8 cstrat;
	
//	if(xpos < 0 || xpos > PlayerMapWidth * 8 || ypos < 0 || ypos > PlayerMapHeight * 8)
//		return; // don't spawn off screen strats
	
	cstrat = current_strat;  // store current_strat so that we can spawn from within the current_strat
	if(num_strats < MAX_STRATS)
	{
		current_strat = num_strats;
		if(Strat_Number >= 0)
			Object[num_strats].stratnum = Strat_Number; // where in the strat list (-1 = not in the list)
		else
			Object[num_strats].stratnum = 255; // not in list
			
		Object[num_strats].handle = NoCollision;
		Object[num_strats].parameter =  parameter;
		parm_xpos = xpos;
		parm_ypos = ypos;
		if(stratinit != 0)
			stratinit();
			
		num_strats++;
	}
	current_strat = cstrat;
}

void handle_strats(void)
{
	nxtenemy = 0;
	SpriteCacheCount = 0;
	if(flash_time > 0)
	{
		flash_time--;
		if(flash_time == 0)
		{
			*flash_ptr = flash_old_bgr;
		}
	}
	WallCollision(player.xpos,player.ypos);
	CeilingCollision(player.xpos,player.ypos - 16);
	for(current_strat = 0 ; current_strat < num_strats; current_strat++)
	{
		Object[current_strat].handle();
//		if(Sprite[player.frame].VPos == 161)
//		{
			// warn sprite zapped
//			warnobject = current_strat;
//		}
	}
}

// set an object's handle to this routine to make it delete itself
void delete_strat(void)
{
	s16 b;
	
	b = num_strats - 1;
	if(b < 0)
		b = 0;
	if(current_strat < b)
		Object[current_strat] = Object[b];
	num_strats--;
	current_strat--;		
}


// call this function before removing a strat to make it respawn if the player dies
void respawnme(void)
{
	if(Object[current_strat].stratnum != 255) // don't respawn if it wasn't map spawned
	{
		respawnlist[numrespawns++] = Object[current_strat].stratnum;
	}
}

// call this function to make the current strat delete itself..
void remove(void) 
{
	if(Object[current_strat].remover != 0)
		Object[current_strat].remover(); // let strat tidy itself up before leaving
	Object[current_strat].handle = delete_strat; // delete it from the strat list
}

void removeall(void)
{
	for(current_strat = 0; current_strat < num_strats;current_strat++)
		if(Object[current_strat].remover != 0)
			Object[current_strat].remover();
    DmaArrayClear(3,161,             Sprite,         32);  // Move undisplayed OBJ to outside of the screen
	num_strats = 0;
	current_strat = 0;
}

// return next available sprite for use by a strat
u8 GrabASprite(void)
{
	u8 a;
	for(a = 3 ; a < 64; a++) // 96 to 127 are used by particle system, 64-95 are dynamic
	{
		if(Sprite[a].VPos == 161)
		{
			Sprite[a].VPos = 160;
			Sprite[a].AffineMode = 0;
			return a;
		}
	}
	
	OutOfSpritesWarn = 1;
	return 63; // no sprite available. we'll return error messages at some point.
}

s16 sgn(s16 val)
{
	if(val < 0)
		return -1;
	else
		if(val > 0)
			return 1;
	return 0;
}

u16 getcollision(u16 object_xpos, u16 object_ypos)
{
	s16 col;
	u16 tx;
	u8 a;	
	tx = object_xpos >> 3;
	
	a = 0;
	while (a < 16 && ((col_map[tx * 16 + a] & 1023) < (object_ypos - 24)))
		a+=2;  // only want the even ones (floor collisions)
	
	if(a == 16)
		return 32767; // bloody great hole in the floor
	collision_floornum = a; // used for wall collisions
	col = col_map[tx * 16 + a] & 1023; // find base tile height of collision (in pixels)

// get fine collision pos
	if(  ((col_map[tx * 16 + a] >> 10) & 63) > 3)
	{
		col++;
	}
	
	col += *(col_tile_fine + ((( col_map[tx * 16 + a] >> 10) & 63) * 8) + (object_xpos & 7));

//	if(col >= PlayerMapHeight * 8)
//		return 32767;
		
	return col - 16;
}

u16 getcollisiondepth(u16 object_xpos,s16 col2) // returns thickness of last collision checked
{
	s16 col;
	u8 a;
	u16 tx;
	
	tx = object_xpos >> 3;
	
	a = collision_floornum + 1;
	
	if(a >= 16)
		return 32767;
		
	col = col_map[tx * 16 + a] & 1023;
// get fine collision pos
	if(  ((col_map[tx * 16 + a] >> 10) & 63) > 3)
	{
		col++;
	}
	
	col += *(col_tile_fine + ((( col_map[tx * 16 + a] >> 10) & 63) * 8) + (object_xpos & 7));
	
	col -= (col2 + 16);
	return col;

}

u16 getwall(u16 object_xpos, u16 object_ypos)
{
	s16 col;
	u16 tx;
	u8 a;
	
	tx = object_xpos >> 3;

	if(object_ypos <= 0)
		object_ypos = 2; 
	a = 0; 
	while(a < 16 && ((col_map[tx * 16 + a] & 1023) < object_ypos))
		a += 2; // floors only first...
		
	
	if(a == 0) // no floor below. so return no wall
		return 0;
		
	// now we're at the floor below the point, so back up one
	a -= 2;
	
	// now get the ceiling for it. if it's > object_ypos, point is in a wall
	
	if((col_map[tx * 16 + a + 1] & 1023) > object_ypos)	
		return 1;
	return 0;
	
}

// call ForceCollision(xpos,ypos) early in the player pass, (before the platform objects are called)
// then platform objects override collision_underme with their floor height if player is over them.

void ForceCollision(u16 object_xpos,u16 object_ypos)
{
	
	collision_underme = getcollision(object_xpos,object_ypos);
	
	
	if(platform_underme >= player.ypos && platform_underme < collision_underme)
		collision_underme = platform_underme;
		
	platform_underme2 = platform_underme;
	platform_underme = 32767;
}

// ForceOtherCollision is used by strats other than the player to get a ground collision (ignores platforms)

s16 ForceOtherCollision(u16 object_xpos,u16 object_ypos)
{
	return getcollision(object_xpos,object_ypos); // find base tile height of collision (in pixels)

}

u16 getceiling(u8 floornum, s16 object_xpos)
{
	u16 tx;
	s16 col; 
	
	tx = object_xpos >> 3;
	
	if(floornum < 16)
	{
		col = col_map[tx * 16 + floornum] & 1023;
		col += *(col_tile_fine + ((( col_map[tx * 16 + floornum] >> 10) & 63) * 8) + (object_xpos & 7));
		return col;
	}
	return 32767;
	
}

void CeilingCollision(u16 object_xpos,u16 object_ypos)
{
	s16 col;
	u16 tx;
	s8 a;	
	tx = object_xpos >> 3;
	
	a = 15;
	while (a > 0 && ((col_map[tx * 16 + a] & 1023) > object_ypos))
		a-=2;  // only want the odd ones (ceiling collisions)
	if(a <= 0)
	{
		col = 0; // bloody great hole in the sky
	}
	else
	{
		col = col_map[tx * 16 + a] & 1023; // find base tile height of collision (in pixels)

// get fine collision pos
	
		col += *(col_tile_fine + ((( col_map[tx * 16 + a] >> 10) & 63) * 8) + (object_xpos & 7));
	}
	collision_aboveme = col;
	col = ForceOtherCollision(object_xpos,collision_aboveme - 32);
	if(col < collision_aboveme && col > collision_aboveme - 48)
		collision_aboveme = 0;
}

void WallCollision(u16 object_xpos,u16 object_ypos)
{
	u16 tx,ty;
	
// do collision underme stuff for player
	
	collision_underme = getcollision(object_xpos,object_ypos);
	Floor_Depth = getcollisiondepth(object_xpos, collision_underme);

	if(platform_underme >= (player.ypos - 2) && platform_underme < collision_underme && platform_underme > 0)
		collision_underme = platform_underme;

	
	platform_underme2 = platform_underme;
	platform_underme = 32767;


// now find collision to left

	collision_left = getwall(object_xpos - 16,max(8,object_ypos - 16));
	collision_left_ladder = getwall(object_xpos - 16,max(8,object_ypos));
// now find collision to right

	collision_right = getwall(object_xpos + 8,max(8,object_ypos - 16) );
	collision_right_ladder = getwall(object_xpos + 8,max(8,object_ypos));
}

s32 min(s32 a,s32 b)
{
	if(a < b)
		return a;
	return b;
}

s32 max(s32 a,s32 b)
{
	if(a < b)
		return b;
	return a;
}

s32 smin(s32 a,u32 b)
{
	if(a < 0)
		return -min(abs(a),b);
	return min(abs(a),b);
}

u32 abs(s32 a)
{
	if(a < 0)
		return (u32)-a;
	return (u32)a;
}

u32 rnd(u32 a)
{
	UpdateRandomClock();
	return RandomClock % a;
	
}

s32 srnd(u32 a)
{
	s32 b;
	UpdateRandomClock();
	b = (RandomClock >> 1) % a;
	if(RandomClock & 1)
		return -b;
	return b;
}

u32 rndfast(u32 a)
{
	UpdateRandomClock();
	return RandomClock & a;
}

void UpdateRandomClock(void)
{
	RandomClock = (RandomClock + (u32)(tiles_le_wahi1[(RandomClock2 + 1) & 0xfff] + 67));
	RandomClock2 = (RandomClock2 + 1) & 0x7ff;
}

void SpriteRotate(u8 myaffine , u8 ang)
{
	u8 aff;
	aff = myaffine << 2;
	Sprite[aff].AffineParam = fsin[(ang + 64) & 255];
	Sprite[aff + 1].AffineParam = fsin[ang];
	Sprite[aff + 2].AffineParam = -fsin[ang];
	Sprite[aff + 3].AffineParam = fsin[(ang + 64) & 255];
}

void SpriteScale(u8 myaffine, s8 scalex,s8 scaley)
{
	u8 aff;
	aff = myaffine << 2;
	Sprite[aff].AffineParam = 1024 / scalex;
	Sprite[aff + 1].AffineParam = 0;
	Sprite[aff + 2].AffineParam = 0;
	Sprite[aff + 3].AffineParam = 1024 / scaley;
	
}


void SpriteScaleFine(u8 myaffine, s16 scalex,s16 scaley)
{
	u8 aff;
	aff = myaffine << 2;
	Sprite[aff].AffineParam = 16384 / scalex;
	Sprite[aff + 1].AffineParam = 0;
	Sprite[aff + 2].AffineParam = 0;
	Sprite[aff + 3].AffineParam = 16384 / scaley;

}

void TSpriteScaleFine(u8 myaffine, s16 scalex,s16 scaley)
{
	u8 aff;
	aff = myaffine << 2;
	TSprite[aff].AffineParam = 16384 / scalex;
	TSprite[aff + 1].AffineParam = 0;
	TSprite[aff + 2].AffineParam = 0;
	TSprite[aff + 3].AffineParam = 16384 / scaley;

}

void SpriteRotateScale(u8 myaffine, u8 ang, s8 scalex,s8 scaley)
{
	u8 aff;
	aff = myaffine << 2;
	
	Sprite[aff].AffineParam = (fsin[(ang + 64) & 255] << 2) / scalex;
	Sprite[aff + 1].AffineParam = (fsin[ang] << 2) / scalex;
	Sprite[aff + 2].AffineParam = (-fsin[ang] << 2) / scaley;
	Sprite[aff + 3].AffineParam = (fsin[(ang + 64) & 255] << 2) / scaley;

}

void CycleColour(u16 palette,u16 start,u16 length)
{
	u16 temp;
	u16 * ptr;
	u8 a;
	
	ptr = (u16 *)(OBJ_PLTT + palette * 32 + start * 2);
	temp = *ptr;
	for(a = 0; a < length ; a++)
		ptr[a] = ptr[a + 1];
	
//	DmaCopy(3,(u32)(ptr + 2),(u32)ptr,length * 2,16);	
//	DmaWait(3);
	ptr[a - 1] = temp; 
}

void CycleMapColour(u16 palette,u16 start,u16 length)
{
	u16 temp;
	u16 * ptr;
	u8 a;
	
	ptr = (u16 *)(BG_PLTT + palette * 32);
	temp = ptr[start + length - 1];
	for(a = start + length - 1; a > start ; a--)
		ptr[a] = ptr[a - 1];
	
//	DmaCopy(3,(u32)(ptr + 2),(u32)ptr,length * 2,16);	
//	DmaWait(3);
	ptr[start] = temp; 
}

void CycleMapColourB(u16 palette,u16 start,u16 length)
{
	u16 temp;
	u16 * ptr;
	u8 a;
	
	ptr = (u16 *)(BG_PLTT + palette * 32 + start * 2);
	temp = *(ptr + (length - 1 ) * 2);
	for(a = length - 1; a > 0 ; a--)
		ptr[a] = ptr[a - 1];
	
//	DmaCopy(3,(u32)(ptr + 2),(u32)ptr,length * 2,16);	
//	DmaWait(3);
	ptr[0] = temp; 
}



void LoadCharAvail(void)
{
	u8 a;
	u16 check;
	u16 result;
		
	memcardinit();
	result = memcardread(16,2); // 64 bytes from slot 2
	memcarddone();

	if(result)
	{
		// error
		MemCardError(result);
	}
	else
	{
		check = 7;
		for(a = 0; a < 11;a++)
			check += SaveGame[a];
			
		if(check != SaveGame[11]) // failed
		{
			CharAvail[0] = 1;
			NCharAvail[0] = 1;
			for(a = 1; a < 6;a++)
			{
				CharAvail[a] = 0;
				NCharAvail[a] = 0;
			}
			SFX_Volume = 2;
			Music_Volume = 2;
			Game_Controls = 0;
			GameSlot[0] = 0;
			GameSlot[1] = 0;
			
		}
		else
		{

			for(a = 0; a < 6;a++)
				CharAvail[a] = SaveGame[a];
			SFX_Volume = SaveGame[6];
			Music_Volume = SaveGame[7];
			Game_Controls = SaveGame[8];
			GameSlot[0] = SaveGame[9];
			GameSlot[1] = SaveGame[10];
		}
	}
}

u16 SaveCharAvail(void)
{
	u8 a;
	u16 check;
	u16 result;
	if(Cheatmode == 4) // wipe cart
	{
		CharAvail[0] = 1;
		NCharAvail[0] = 1;
		for(a = 1; a < 6;a++)
		{
			CharAvail[a] = 0;
			NCharAvail[a] = 0;
		}
		SFX_Volume = 2;
		Music_Volume = 2;
		Game_Controls = 0;
		GameSlot[0] = 0;
		GameSlot[1] = 0;
	}
		
	for(a = 0; a < 6; a++)
		SaveGame[a] = CharAvail[a];
		
	SaveGame[6] = SFX_Volume;
	SaveGame[7] = Music_Volume;
	SaveGame[8] = Game_Controls;
	SaveGame[9] = GameSlot[0];
	SaveGame[10] = GameSlot[1];
	
	check = 7;
	for(a = 0; a < 11;a++)
		check += SaveGame[a];
	SaveGame[11] = check;
	memcardinit();
	result = memcardwrite(16,2); // 64 bytes into slot 2
	memcarddone();
	return result;


}

void SaveGameParameters(void)
{
	u8 a;
	u16 check;
	SaveGame[0] = player.lives;
	SaveGame[1] = player.elemental_power;
	SaveGame[2] = Game_Character;
	SaveGame[3] = Game_Level;
	SaveGame[4] = Frisbee_Power;
	SaveGame[5] = Game_Difficulty;
	check = 0;
	for(a = 0; a < 6;a++)
		check = check + SaveGame[a];
	SaveGame[6] = check;
}

void LoadGameParameters(void)
{
	player.lives = SaveGame[0];
	player.elemental_power = SaveGame[1];
	Game_Character = SaveGame[2];
	Game_Level = SaveGame[3];
	Frisbee_Power = SaveGame[4];
	Game_Difficulty = SaveGame[5];
	
}




void PasteSprite(s16 xpos,s16 ypos,u8 shape,u8 size,u16 charno)
{
    u8 me;
    me = GrabASprite();
    
    Sprite[me] = *(OamData *)OamData_Sample;
    Sprite[me].Priority = 0;
    Sprite[me].ObjMode = 0;
	Sprite[me].Shape = shape;
	Sprite[me].Size = size;
	Sprite[me].CharNo = charno;
	Sprite[me].Pltt = 0;
	Sprite[me].VPos = ypos;
	Sprite[me].HPos = xpos;

}

void CheckSpriteCache(u32 x,u32 z)
{
	u8 a;
//#if 0
	for(a = 0; a < nxtenemy; a++)
		if(x == SpriteCacheSource[a])
		{
			SpriteCache = a;
			return;
		}
//#endif
	
	SpriteCache = nxtenemy;
	if(nxtenemy < MaxEnemies)
		nxtenemy++;
		
//	nxtenemy = (nxtenemy + 1) % MaxEnemies;
//	SpriteCacheCount++;
//	if(SpriteCacheCount > SpriteCacheOverrun)
//		SpriteCacheOverrun = SpriteCacheCount;
	SpriteCacheSource[SpriteCache] = x;
	SpriteCacheSize[SpriteCache] = z;
	
//	DmaCopy(3,x,(u32)(OBJ_MODE0_VRAM + EnemySpriteOffset[SpriteCache]),z,16);  
//	DmaWait(3)	
//	return 1;
}

void CheckPalCache(u32 x)
{
	u8 a;
#if 1	
	for(a = 0; a < nxtenemypal; a++)
	{
		if(x == PalCacheSource[a])
		{
			PalCache = a;
			return;
		}
	}
#endif		

	PalCache = nxtenemypal;
	if(nxtenemypal < MaxEnemyPals)
		nxtenemypal++;
//	nxtenemypal = (nxtenemypal + 1) % MaxEnemyPals;
	
	PalCacheSource[PalCache] = x;

/*
	if(nxtenemypal < MaxEnemyPals)
		PalCache = nxtenemypal++;
	else
		PalCache = MaxEnemyPals - 1;

*/
	return;
	//nxtenemypal = (nxtenemypal + 1) % MaxEnemyPals;
}	

void CyclePalCache(u32 x,u32 y)
{
	PalCacheFlash[PalCache] = x;
	PalCacheFlashOff[PalCache] = y;	
}

void FlushSpriteCache(void)
{
	u8 a;
	for(a = 0; a < 6; a++)
	{
		LastEnemySprite[a] = 0;
		LastEnemyPalette[a] = 0;
		EnemySpriteUsed[a] = 0;
		EnemyPalUsed[a] = 0;
	}
}

void SetMapColour(u8 index,u8 red, u8 green, u8 blue)
{
	u16 bgr;
	u16 * ptr;
	
	bgr = red + (green << 5) + (blue << 10);
	ptr = BG_PLTT + index * 2;
	*ptr = bgr;
}

void FlashMapColour(u8 index,u8 red,u8 green,u8 blue)
{
	u16 bgr;
	
	if(flash_time > 0)
	{
		*flash_ptr = flash_old_bgr;
	}
	bgr = red + (green << 5) + (blue << 10);
	flash_ptr = (BG_PLTT + index * 2);
	
	flash_old_bgr = *flash_ptr;
	*(vu16 *)flash_ptr = bgr;

	flash_time = 5;
}

void DimMapPalette(u32 pal)
{
	u16 a;
	u16 bgr;
	u16 * ptr;
	
	bgr = (15 << 5) + (15 << 10) + 15;
	
	ptr = BG_PLTT + 32;
	for(a = 0; a < 240;a ++)
	{
		*(vu16 *)ptr = (*(vu16 *)ptr++) & bgr;	
	}
}
