//////////////////////////////////////////////////////////////////////////////
// main.cpp                                                                 //
//////////////////////////////////////////////////////////////////////////////
/*
	Simple particle emitter using all 128 sprites (8x8)
*/

//////////////////////////////////////////////////////////////////////////////
// Includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <agb.h>
#include <stdio.h>
#include <debug.h>
#include <stdlib.h>
#include "res.h"

//////////////////////////////////////////////////////////////////////////////
// Typedefs                                                                  //
//////////////////////////////////////////////////////////////////////////////
struct Particle
{
	int x, y, z;			// current position
	int ix, iy, iz;			// velocity
	int iix, iiy, iiz;		// gravity
	int time;				// how long the particle lived (used for selecting the tile)
};

typedef int BOOL;

//////////////////////////////////////////////////////////////////////////////
// Variables                                                                //
//////////////////////////////////////////////////////////////////////////////
struct Particle particles[128];
BOOL quit = FALSE;

//////////////////////////////////////////////////////////////////////////////
// VBlankIntr                                                               //
//////////////////////////////////////////////////////////////////////////////
void VBlankIntr()
{
	KeyData k = (KeyData &)rKeyData;
	if (!k.Start || (getchar() == 27)) quit = true;		// exit on start or escape
	
	rINTR_CHECK_BUF = V_BLANK_INTR_FLAG;     // Set V Blank interrupt check
}

//////////////////////////////////////////////////////////////////////////////
// ResetParticle                                                            //
//////////////////////////////////////////////////////////////////////////////
void ResetParticle(Particle &p)
{
	// just make some initialisation delay
	p.x = ((rand()%240)-120) << 8; p.y = (rand()%160) << 8; p.z = 0;
	p.ix = 0; p.iy = rand()%800; p.iz = 0;
	p.iix = 0; p.iiy = -1; p.iiz = 0;
	p.time = 99999999;
}

//////////////////////////////////////////////////////////////////////////////
// EmitParticle                                                             //
//////////////////////////////////////////////////////////////////////////////
void EmitParticle(Particle &p)
{
	// emit new particle
	p.x = 0; p.y = 0; p.z = 0;		// origin
	p.ix = (rand()%50)-25; p.iy = 200 + (rand()%50); p.iz = (rand()%50)-25;	
	p.iix = 0; p.iiy = -1; p.iiz = 0;
	p.time = 0;
}

//////////////////////////////////////////////////////////////////////////////
// main                                                                     //
//////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
	printf("Particle sprites\n");

	// init interrupts	
	iVBlankIntr = VBlankIntr;
	rIME = 0;
	rIE |= V_BLANK_INTR_FLAG;
	rIME = 1;
	rSTAT = STAT_V_BLANK_IF_ENABLE;

	// set gfx mode
	rDISPCNT = DISP_MODE_2 | DISP_OBJ_CHAR_1D_MAP | DISP_BG2_ON | DISP_OBJ_ON;
	rBG2CNT = BG_LOOP_ON | BG_MOS_OFF | BG_COLOR_256 | BG_PRIORITY_0 | BG_SCREEN_SIZE_0 | 0<<BG_CHAR_BASE_SHIFT | 31<<BG_SCREEN_BASE_SHIFT;

	// init gfx data
	DmaArrayCopy(3, tiles1_pal, obj_palette, 16);
	DmaArrayCopy(3, tiles1_256, (void *)(OBJ_MODE2_VRAM + 64*0), 16);

	// init common sprite data
	OamData o;
	o.AffineMode = ST_OAM_AFFINE_NONE;
	o.ObjMode = ST_OAM_OBJ_NORMAL;
	o.Mosaic = 0;
	o.ColorMode = ST_OAM_COLOR_256;
	o.Shape = ST_OAM_SQUARE;
	o.AffineParamNo_L = 0;	// affine nr. bit 0..2
	o.HFlip = 0;			// affine nr. bit 3
	o.VFlip = 0;			// affine nr. bit 4
	o.Size = 0;
	o.CharNo = 0;
	o.Priority = 0;
	o.Pltt = 0;			// unused in 256 color mode

	// reset particles
	for (int i=0; i<128; i++)
	{
		oamData[i] = o;			// copy common sprite data
		ResetParticle(particles[i]);
	}

	// main loop
	while (!quit)
	{
		for (int i=0; i<128; i++)	// do all sprites
		{
			// update particle
			Particle &p = particles[i];
			p.x += p.ix; p.y += p.iy; p.z += p.iz;
			p.ix += p.iix; p.iy += p.iiy; p.iz += p.iiz;

			OamData o = oamData[i];
			// update sprite
			o.VPos = 160 - (p.y>>8);
			o.HPos = (p.x>>8) + 120;
			oamData[i] = o;
			int blah = (1<<9) - p.time++;
			if (blah < 0) blah = 0;
			oamData[i].CharNo = blah << 4 >> 9 << 1;		// << 1 for 256-color mode tilenumber!
			
			// re-emit when touched ground
			if (p.y < 0) EmitParticle(particles[i]);
		}
		
		//VBlankIntrWait();
	}

	printf("bye!\n");
	
	return 0;
}
